@@ -18,9 +18,6 @@
  *    along with this program; if not, write to the Free Software
  *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
-#ifdef HAVE_CVS_IDENT
-#ident "$Id: fpga_priv.h,v 1.8 2003/07/02 00:48:03 steve Exp $"
-#endif
 
 # include  <stdio.h>
 # include  "device.h"
@@ -51,34 +48,4 @@ extern void xnf_mangle_lpm_name(ivl_lpm_t net, char*buf, size_t nbuf);
 extern const char*xnf_mangle_nexus_name(ivl_nexus_t net);
 
 
-/*
- * $Log: fpga_priv.h,v $
- * Revision 1.8  2003/07/02 00:48:03  steve
- *  No longer export generic-edif functions.
- *
- * Revision 1.7  2003/06/24 03:55:00  steve
- *  Add ivl_synthesis_cell support for virtex2.
- *
- * Revision 1.6  2002/08/12 01:35:03  steve
- *  conditional ident string using autoconfig.
- *
- * Revision 1.5  2002/08/11 23:47:04  steve
- *  Add missing Log and Ident strings.
- *
- * Revision 1.4  2001/09/06 04:28:40  steve
- *  Separate the virtex and generic-edif code generators.
- *
- * Revision 1.3  2001/09/02 21:33:07  steve
- *  Rearrange the XNF code generator to be generic-xnf
- *  so that non-XNF code generation is also possible.
- *
- *  Start into the virtex EDIF output driver.
- *
- * Revision 1.2  2001/08/30 04:31:04  steve
- *  Mangle nexus names.
- *
- * Revision 1.1  2001/08/28 04:14:20  steve
- *  Add the fpga target.
- *
- */
 #endif
