@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001-2009 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2001-2010 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -23,13 +23,10 @@
 #ifdef CHECK_WITH_VALGRIND
 # include  "vvp_cleanup.h"
 #endif
-# include  <stdio.h>
-#ifdef HAVE_MALLOC_H
-# include  <malloc.h>
-#endif
-# include  <stdlib.h>
-# include  <string.h>
-# include  <assert.h>
+# include  <cstdio>
+# include  <cstdlib>
+# include  <cstring>
+# include  <cassert>
 
 static int string_get(int code, vpiHandle ref)
 {
@@ -54,7 +51,7 @@ static int string_get(int code, vpiHandle ref)
           case vpiAutomatic:
 	      return 0;
 
-#ifdef CHECK_WITH_VALGRIND
+#if defined(CHECK_WITH_VALGRIND) || defined(BR916_STOPGAP_FIX)
           case _vpiFromThr:
 	      return _vpiNoThr;
 #endif
@@ -167,7 +164,8 @@ static void string_value(vpiHandle ref, p_vpi_value vp)
 
 
 	  default:
-	    fprintf(stderr, "ERROR (vpi_const.cc): vp->format: %d\n", vp->format);
+	    fprintf(stderr, "ERROR (vpi_const.cc): vp->format: %d\n",
+	            (int)vp->format);
 	    assert(0);
 
 	    vp->format = vpiSuppressVal;
@@ -365,7 +363,7 @@ static int binary_get(int code, vpiHandle ref)
           case vpiAutomatic:
 	    return 0;
 
-#ifdef CHECK_WITH_VALGRIND
+#if defined(CHECK_WITH_VALGRIND) || defined(BR916_STOPGAP_FIX)
           case _vpiFromThr:
 	      return _vpiNoThr;
 #endif
@@ -405,7 +403,7 @@ static void binary_value(vpiHandle ref, p_vpi_value vp)
 
 	  default:
 	    fprintf(stderr, "vvp error: format %d not supported "
-		    "by vpiBinaryConst\n", vp->format);
+		    "by vpiBinaryConst\n", (int)vp->format);
 	    vp->format = vpiSuppressVal;
 	    break;
       }
@@ -566,7 +564,7 @@ static int dec_get(int code, vpiHandle ref)
           case vpiAutomatic:
 	    return 0;
 
-#ifdef CHECK_WITH_VALGRIND
+#if defined(CHECK_WITH_VALGRIND) || defined(BR916_STOPGAP_FIX)
           case _vpiFromThr:
 	      return _vpiNoThr;
 #endif
@@ -624,7 +622,7 @@ static void dec_value(vpiHandle ref, p_vpi_value vp)
 
 	  default:
 	    fprintf(stderr, "vvp error (vpi_const.cc): format %d not supported "
-		    "by vpiDecConst\n", vp->format);
+		    "by vpiDecConst\n", (int)vp->format);
 	    vp->format = vpiSuppressVal;
 	    break;
       }
@@ -677,7 +675,7 @@ static int real_get(int code, vpiHandle ref)
           case vpiAutomatic:
 	    return 0;
 
-#ifdef CHECK_WITH_VALGRIND
+#if defined(CHECK_WITH_VALGRIND) || defined(BR916_STOPGAP_FIX)
           case _vpiFromThr:
 	      return _vpiNoThr;
 #endif
@@ -696,16 +694,7 @@ static void real_value(vpiHandle ref, p_vpi_value vp)
       assert((ref->vpi_type->type_code == vpiConstant) ||
              (ref->vpi_type->type_code == vpiParameter));
 
-      switch (vp->format) {
-	  case vpiObjTypeVal:
-	    vp->format = vpiRealVal;
-	  case vpiRealVal:
-	    vp->value.real = rfp->value;
-	    break;
-	  default:
-	    fprintf(stderr, "vvp error: unsupported format %d.\n", vp->format);
-	    assert(0);
-      }
+       vpip_real_get_value(rfp->value, vp);
 }
 
 static const struct __vpirt vpip_real_rt = {
