@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2004-2009 Stephen Williams (steve@icarus.com)
+ * Copyright (c) 2004-2013 Stephen Williams (steve@icarus.com)
  *
  *    This source code is free software; you can redistribute it
  *    and/or modify it in source code form under the terms of the GNU
@@ -22,20 +22,21 @@
 # include  "vpi_priv.h"
 # include  "schedule.h"
 # include  "statistics.h"
-# include  <stdio.h>
+# include  <cstdio>
 # include  <cstring>
 # include  <cstdlib>
 # include  <iostream>
 # include  <typeinfo>
-# include  <limits.h>
-# include  <math.h>
-# include  <assert.h>
+# include  <climits>
+# include  <cmath>
+# include  <cassert>
 #ifdef CHECK_WITH_VALGRIND
 # include  <valgrind/memcheck.h>
 # include  <map>
+# include "sfunc.h"
 #endif
 
-// Allocate around 1Megbytes/chunk.
+// Allocate around 1Megabytes/chunk.
 static const size_t VVP_NET_CHUNK = 1024*1024/sizeof(vvp_net_t);
 static vvp_net_t*vvp_net_alloc_table = 0;
 #ifdef CHECK_WITH_VALGRIND
@@ -80,19 +81,37 @@ void* vvp_net_t::operator new (size_t size)
 
 #ifdef CHECK_WITH_VALGRIND
 static map<vvp_net_t*, bool> vvp_net_map;
+static map<sfunc_core*, bool> sfunc_map;
 
 void vvp_net_delete(vvp_net_t *item)
 {
-       vvp_net_map[item] = true;
+      vvp_net_map[item] = true;
+      if (sfunc_core*tmp = dynamic_cast<sfunc_core*> (item->fun)) {
+	    sfunc_map[tmp] = true;
+      }
 }
 
 void vvp_net_pool_delete()
 {
+      unsigned long vvp_nets_del = 0;
+
       map<vvp_net_t*, bool>::iterator iter;
       for (iter = vvp_net_map.begin(); iter != vvp_net_map.end(); iter++) {
+	    vvp_nets_del += 1;
 	    VALGRIND_MEMPOOL_FREE(iter->first->pool, iter->first);
       }
 
+      map<sfunc_core*, bool>::iterator siter;
+      for (siter = sfunc_map.begin(); siter != sfunc_map.end(); siter++) {
+	    delete siter->first;
+      }
+
+      if (RUNNING_ON_VALGRIND && (vvp_nets_del != count_vvp_nets)) {
+	    fflush(NULL);
+	    VALGRIND_PRINTF("Error: vvp missed deleting %lu net(s).",
+	                    count_vvp_nets - vvp_nets_del);
+      }
+
       for (unsigned idx = 0; idx < vvp_net_pool_count; idx += 1) {
 	    VALGRIND_DESTROY_MEMPOOL(vvp_net_pool[idx])
 	    ::delete [] vvp_net_pool[idx];
@@ -171,10 +190,10 @@ vvp_bit4_t add_with_carry(vvp_bit4_t a, vvp_bit4_t b, vvp_bit4_t&c)
 	  case 3:
 	    c = BIT4_1;
 	    return BIT4_1;
-	  default:
-	    fprintf(stderr, "Incorrect result %d.\n", sum);
-	    assert(0);
       }
+      fprintf(stderr, "Incorrect result %d.\n", sum);
+      assert(0);
+      return BIT4_X;
 }
 
 vvp_bit4_t scalar_to_bit4(PLI_INT32 scalar)
@@ -188,10 +207,10 @@ vvp_bit4_t scalar_to_bit4(PLI_INT32 scalar)
 	    return BIT4_X;
 	  case vpiZ:
 	    return BIT4_Z;
-	  default:
-	    fprintf(stderr, "Unsupported scalar value %d.\n", scalar);
-	    assert(0);
       }
+      fprintf(stderr, "Unsupported scalar value %d.\n", (int)scalar);
+      assert(0);
+      return BIT4_X;
 }
 
 vvp_bit4_t operator ^ (vvp_bit4_t a, vvp_bit4_t b)
@@ -1106,26 +1125,26 @@ void vvp_vector4_t::invert()
 
 vvp_vector4_t& vvp_vector4_t::operator &= (const vvp_vector4_t&that)
 {
-	// Make sure that all Z bits are turned into X bits.
-      change_z2x();
-
-	// This is sneaky. The truth table is:
-	//     00 01 11
-	//  00 00 00 00
-	//  01 00 01 11
-	//  11 00 11 11
+	// The truth table is:
+	//     00 01 11 10
+	//  00 00 00 00 00
+	//  01 00 01 11 11
+	//  11 00 11 11 11
+	//  10 00 11 11 11
       if (size_ <= BITS_PER_WORD) {
-	      // Each tmp bit is true if that is 1, X or Z.
-	    unsigned long tmp = that.abits_val_ | that.bbits_val_;
-	    abits_val_ &= that.abits_val_;
-	    bbits_val_ = (bbits_val_ & tmp) | (abits_val_&that.bbits_val_);
-
+	    unsigned long tmp1 = abits_val_ | bbits_val_;
+	    unsigned long tmp2 = that.abits_val_ | that.bbits_val_;
+	    abits_val_ = tmp1 & tmp2;
+	    bbits_val_ = (tmp1 & that.bbits_val_) | (tmp2 & bbits_val_);
       } else {
 	    unsigned words = (size_ + BITS_PER_WORD - 1) / BITS_PER_WORD;
 	    for (unsigned idx = 0; idx < words ; idx += 1) {
-		  unsigned long tmp = that.abits_ptr_[idx]|that.bbits_ptr_[idx];
-		  abits_ptr_[idx] &= that.abits_ptr_[idx];
-		  bbits_ptr_[idx] = (bbits_ptr_[idx]&tmp) | (abits_ptr_[idx]&that.bbits_ptr_[idx]);
+		  unsigned long tmp1 = abits_ptr_[idx] | bbits_ptr_[idx];
+		  unsigned long tmp2 = that.abits_ptr_[idx] |
+		                       that.bbits_ptr_[idx];
+		  abits_ptr_[idx] = tmp1 & tmp2;
+		  bbits_ptr_[idx] = (tmp1 & that.bbits_ptr_[idx]) |
+		                    (tmp2 & bbits_ptr_[idx]);
 	    }
       }
 
@@ -1134,34 +1153,29 @@ vvp_vector4_t& vvp_vector4_t::operator &= (const vvp_vector4_t&that)
 
 vvp_vector4_t& vvp_vector4_t::operator |= (const vvp_vector4_t&that)
 {
-	// Make sure that all Z bits are turned into X bits.
-      change_z2x();
-
-	// This is sneaky.
-	// The OR is 1 if either operand is 1.
-	// The OR is 0 if both operants are 0.
-	// Otherwise, the AND is X. The truth table is:
-	//
-	//     00 01 11
-	//  00 00 01 11
-	//  01 01 01 01
-	//  11 11 01 11
+	// The truth table is:
+	//     00 01 11 10
+	//  00 00 01 11 11
+	//  01 01 01 01 01
+	//  11 11 01 11 11
+	//  10 11 01 11 11
       if (size_ <= BITS_PER_WORD) {
-	      // Each tmp bit is true if that is 1, X or Z.
-	    unsigned long tmp1 = abits_val_ | bbits_val_;
-	    unsigned long tmp2 = that.abits_val_ | that.bbits_val_;
-	    bbits_val_ =  (bbits_val_& ~(that.abits_val_^that.bbits_val_))
-		        | (that.bbits_val_& ~abits_val_);
-	    abits_val_ = tmp1 | tmp2;
+	    unsigned long tmp = abits_val_ | bbits_val_ |
+	                        that.abits_val_ | that.bbits_val_;
+	    bbits_val_ = ((~abits_val_ | bbits_val_) & that.bbits_val_) |
+	                 ((~that.abits_val_ | that.bbits_val_) & bbits_val_);
+	    abits_val_ = tmp;
 
       } else {
 	    unsigned words = (size_ + BITS_PER_WORD - 1) / BITS_PER_WORD;
 	    for (unsigned idx = 0; idx < words ; idx += 1) {
-		  unsigned long tmp1 = abits_ptr_[idx] | bbits_ptr_[idx];
-		  unsigned long tmp2 = that.abits_ptr_[idx] | that.bbits_ptr_[idx];
-	    bbits_ptr_[idx] =  (bbits_ptr_[idx]& ~(that.abits_ptr_[idx]^that.bbits_ptr_[idx]))
-		        | (that.bbits_ptr_[idx]& ~abits_ptr_[idx]);
-	    abits_ptr_[idx] = tmp1 | tmp2;
+		  unsigned long tmp = abits_ptr_[idx] | bbits_ptr_[idx] |
+	                        that.abits_ptr_[idx] | that.bbits_ptr_[idx];
+		  bbits_ptr_[idx] = ((~abits_ptr_[idx] | bbits_ptr_[idx]) &
+		                     that.bbits_ptr_[idx]) |
+		                    ((~that.abits_ptr_[idx] |
+		                      that.bbits_ptr_[idx]) & bbits_ptr_[idx]);
+		  abits_ptr_[idx] = tmp;
 	    }
       }
 
@@ -1271,6 +1285,41 @@ bool vector4_to_value(const vvp_vector4_t&vec, unsigned long&val)
 }
 
 #ifndef UL_AND_TIME64_SAME
+bool vector4_to_value(const vvp_vector4_t&vec, int64_t&val,
+		      bool is_signed, bool is_arithmetic)
+{
+      long res = 0;
+      long msk = 1;
+      bool rc_flag = true;
+
+      unsigned size = vec.size();
+      if (size > 8*sizeof(val)) size = 8*sizeof(val);
+      for (unsigned idx = 0 ;  idx < size ;  idx += 1) {
+	    switch (vec.value(idx)) {
+		case BIT4_0:
+		  break;
+		case BIT4_1:
+		  res |= msk;
+		  break;
+		default:
+		  if (is_arithmetic)
+			return false;
+		  else
+			rc_flag = false;
+	    }
+
+	    msk <<= 1L;
+      }
+
+      if (is_signed && vec.value(vec.size()-1) == BIT4_1) {
+	    if (vec.size() < 8*sizeof(val))
+		  res |= (-1L) << vec.size();
+      }
+
+      val = res;
+      return rc_flag;
+}
+
 bool vector4_to_value(const vvp_vector4_t&vec, vvp_time64_t&val)
 {
       vvp_time64_t res = 0;
@@ -1349,6 +1398,10 @@ vvp_realarray_t::vvp_realarray_t(unsigned wor)
 : words_(wor)
 {
       array_ = new double[words_];
+	// Real array words have a default value of zero.
+      for (unsigned idx = 0 ; idx < words_; idx += 1) {
+	    array_[idx] = 0.0;
+      }
 }
 
 vvp_realarray_t::~vvp_realarray_t()
@@ -1523,8 +1576,8 @@ void vvp_vector4array_aa::reset_instance(vvp_context_t context)
 	    for (unsigned idx = 0 ; idx < words_ ; idx += 1) {
 		  if (cell->abits_ptr_) {
                         for (unsigned n = 0 ; n < cnt ; n += 1) {
-                  	    cell->abits_ptr_[n] = vvp_vector4_t::WORD_X_ABITS;
-                  	    cell->bbits_ptr_[n] = vvp_vector4_t::WORD_X_BBITS;
+			      cell->abits_ptr_[n] = vvp_vector4_t::WORD_X_ABITS;
+			      cell->bbits_ptr_[n] = vvp_vector4_t::WORD_X_BBITS;
                         }
                   }
                   cell++;
@@ -2279,12 +2332,11 @@ ostream& operator<< (ostream&out, const vvp_vector2_t&that)
 vvp_vector8_t::vvp_vector8_t(const vvp_vector8_t&that)
 {
       size_ = that.size_;
-      if (size_ <= PTR_THRESH) {
+      if (size_ <= sizeof(val_)) {
 	    memcpy(val_, that.val_, sizeof(val_));
       } else {
-	    ptr_ = new vvp_scalar_t[size_];
-	    for (unsigned idx = 0 ;  idx < size_ ;  idx += 1)
-		  ptr_[idx] = that.ptr_[idx];
+	    ptr_ = new unsigned char[size_];
+	    memcpy(ptr_, that.ptr_, size_);
       }
 }
 
@@ -2295,25 +2347,25 @@ vvp_vector8_t::vvp_vector8_t(const vvp_vector4_t&that,
       if (size_ == 0)
 	    return;
 
-      vvp_scalar_t*tmp;
-      if (size_ <= PTR_THRESH)
-	    tmp = new (val_) vvp_scalar_t[PTR_THRESH];
-      else
-	    tmp = ptr_ = new vvp_scalar_t[size_];
-
-      for (unsigned idx = 0 ;  idx < size_ ;  idx += 1)
-	    tmp[idx] = vvp_scalar_t (that.value(idx), str0, str1);
-
+      if (size_ <= sizeof(val_)) {
+	    ptr_ = 0; // Prefill all val_ bytes
+	    for (unsigned idx = 0 ; idx < size_ ; idx += 1)
+		  val_[idx] = vvp_scalar_t(that.value(idx),str0, str1).raw();
+      } else {
+	    ptr_ = new unsigned char[size_];
+	    for (unsigned idx = 0 ;  idx < size_ ;  idx += 1)
+		  ptr_[idx] = vvp_scalar_t(that.value(idx), str0, str1).raw();
+      }
 }
 
 vvp_vector8_t& vvp_vector8_t::operator= (const vvp_vector8_t&that)
 {
 	// Assign to self.
-      if (size_ > PTR_THRESH && that.size_ > PTR_THRESH && ptr_ == that.ptr_)
+      if (this == &that)
 	    return *this;
 
       if (size_ != that.size_) {
-	    if (size_ > PTR_THRESH)
+	    if (size_ > sizeof(val_))
 		  delete[]ptr_;
 	    size_ = 0;
       }
@@ -2323,7 +2375,7 @@ vvp_vector8_t& vvp_vector8_t::operator= (const vvp_vector8_t&that)
 	    return *this;
       }
 
-      if (that.size_ <= PTR_THRESH) {
+      if (that.size_ <= sizeof(val_)) {
 	    size_ = that.size_;
 	    memcpy(val_, that.val_, sizeof(val_));
 	    return *this;
@@ -2331,11 +2383,10 @@ vvp_vector8_t& vvp_vector8_t::operator= (const vvp_vector8_t&that)
 
       if (size_ == 0) {
 	    size_ = that.size_;
-	    ptr_ = new vvp_scalar_t[size_];
+	    ptr_ = new unsigned char[size_];
       }
 
-      for (unsigned idx = 0 ;  idx < size_ ;  idx += 1)
-	    ptr_[idx] = that.ptr_[idx];
+      memcpy(ptr_, that.ptr_, size_);
 
       return *this;
 }
@@ -2344,12 +2395,12 @@ vvp_vector8_t vvp_vector8_t::subvalue(unsigned base, unsigned wid) const
 {
       vvp_vector8_t tmp (wid);
 
-      vvp_scalar_t* tmp_ptr = tmp.size_<=PTR_THRESH? reinterpret_cast<vvp_scalar_t*>(tmp.val_) : tmp.ptr_;
-      const vvp_scalar_t* ptr = size_<=PTR_THRESH? reinterpret_cast<const vvp_scalar_t*>(val_) : ptr_;
+      unsigned char* tmp_ptr = tmp.size_ <= sizeof(val_) ? tmp.val_ : tmp.ptr_;
+      const unsigned char*use_ptr = size_ <= sizeof(val_) ? val_ : ptr_;
 
       unsigned idx = 0;
       while ((idx < wid) && (base+idx < size_)) {
-	    tmp_ptr[idx] = ptr[base+idx];
+	    tmp_ptr[idx] = use_ptr[base+idx];
 	    idx += 1;
       }
 
@@ -2361,8 +2412,10 @@ vvp_vector8_t part_expand(const vvp_vector8_t&that, unsigned wid, unsigned off)
       assert(off < wid);
       vvp_vector8_t tmp (wid);
 
-      vvp_scalar_t* tmp_ptr = tmp.size_<=vvp_vector8_t::PTR_THRESH? reinterpret_cast<vvp_scalar_t*>(tmp.val_) : tmp.ptr_;
-      const vvp_scalar_t* that_ptr = that.size_<=vvp_vector8_t::PTR_THRESH? reinterpret_cast<const vvp_scalar_t*>(that.val_) : that.ptr_;
+      unsigned char* tmp_ptr = tmp.size_<= sizeof(tmp.val_) ?
+                               tmp.val_ : tmp.ptr_;
+      const unsigned char* that_ptr = that.size_<= sizeof(that.val_) ?
+                                      that.val_ : that.ptr_;
 
       unsigned idx = off;
 
@@ -2725,6 +2778,10 @@ void vvp_fun_signal4_sa::calculate_output_(vvp_net_ptr_t ptr)
 
 void vvp_fun_signal4_sa::release(vvp_net_ptr_t ptr, bool net)
 {
+	// If not currently forced, do nothing.
+      if (force_mask_.size() == 0)
+            return;
+
       force_mask_ = vvp_vector2_t();
       if (net) {
 	    vvp_send_vec4(ptr.ptr()->out, bits4_, 0);
@@ -2737,11 +2794,17 @@ void vvp_fun_signal4_sa::release(vvp_net_ptr_t ptr, bool net)
 void vvp_fun_signal4_sa::release_pv(vvp_net_ptr_t ptr, bool net,
                                     unsigned base, unsigned wid)
 {
+	// If not currently forced, do nothing.
+      if (force_mask_.size() == 0)
+            return;
+
       assert(bits4_.size() >= base + wid);
 
       for (unsigned idx = 0 ;  idx < wid ;  idx += 1) {
-	    force_mask_.set_bit(base+idx, 0);
-	    if (!net) bits4_.set_bit(base+idx, force_.value(base+idx));
+            if (force_mask_.value(base+idx)) {
+                  force_mask_.set_bit(base+idx, 0);
+                  if (!net) bits4_.set_bit(base+idx, force_.value(base+idx));
+            }
       }
       if (force_mask_.is_zero()) force_mask_ = vvp_vector2_t();
 
@@ -3020,6 +3083,10 @@ void vvp_fun_signal8::calculate_output_(vvp_net_ptr_t ptr)
 
 void vvp_fun_signal8::release(vvp_net_ptr_t ptr, bool net)
 {
+	// If not currently forced, do nothing.
+      if (force_mask_.size() == 0)
+            return;
+
       force_mask_ = vvp_vector2_t();
       if (net) {
 	    vvp_send_vec8(ptr.ptr()->out, bits8_);
@@ -3032,11 +3099,17 @@ void vvp_fun_signal8::release(vvp_net_ptr_t ptr, bool net)
 void vvp_fun_signal8::release_pv(vvp_net_ptr_t ptr, bool net,
                                  unsigned base, unsigned wid)
 {
+	// If not currently forced, do nothing.
+      if (force_mask_.size() == 0)
+            return;
+
       assert(bits8_.size() >= base + wid);
 
       for (unsigned idx = 0 ;  idx < wid ;  idx += 1) {
-	    force_mask_.set_bit(base+idx, 0);
-	    if (!net) bits8_.set_bit(base+idx, force_.value(base+idx));
+            if (force_mask_.value(base+idx)) {
+                  force_mask_.set_bit(base+idx, 0);
+                  if (!net) bits8_.set_bit(base+idx, force_.value(base+idx));
+            }
       }
       if (force_mask_.is_zero()) force_mask_ = vvp_vector2_t();
 
@@ -3141,6 +3214,10 @@ void vvp_fun_signal_real_sa::recv_real(vvp_net_ptr_t ptr, double bit,
 
 void vvp_fun_signal_real_sa::release(vvp_net_ptr_t ptr, bool net)
 {
+	// If not currently forced, do nothing.
+      if (force_mask_.size() == 0)
+            return;
+
       force_mask_ = vvp_vector2_t();
       if (net) {
 	    vvp_send_real(ptr.ptr()->out, bits_, 0);
